@@ -6,29 +6,25 @@
 
 using namespace arma;
 
-
-template< class T>
-void print_vec(const std::vector<T>& v)
+mat mat_init(const mat& orig)
 {
-	const int n=v.size();
-	int r=0;
-	printf("[1]   ");
-	for(int i=0;i<n;i++)
-	{
-		printf("%012.3f     ", (double)v[i]);
-		r++;
-		if(r==7)
-		{
-			r=0;
-			printf("\n[%i]  ",i);
-		}
-	}
-	printf("\n");
-	fflush(stdout);
+	return mat(orig.n_rows, orig.n_cols, fill::zeros);
+}
+
+vec vec_init(const vec& orig)
+{
+	return vec(orig.n_elem, fill::zeros);
 }
 
 
+#pragma omp declare reduction( + : arma::mat : omp_out += omp_in ) \
+initializer( omp_priv = mat_init(omp_orig) )
 
+#pragma omp declare reduction( + : arma::vec : omp_out += omp_in ) \
+initializer( omp_priv = vec_init(omp_orig) )
+
+
+// to accomodate early finish routing rules, g should be max(a) longer than the maximum scoe in a booklet
 int elsym(const int routing, const vec& b, const ivec& a, 
 			int* first_ptr, int* last_ptr, const int nI,
 			const int* mod_min_ptr, const int* mod_max_ptr, const int nmod,
@@ -50,7 +46,7 @@ int elsym(const int routing, const vec& b, const ivec& a,
 	for(int m=0; m<nmod; m++)
 		cnit[m+1] = cnit[m] + nit[m];	
 	
-	std::vector<long double> g(g_out.size(), 0); 
+	std::vector<long double> g(g_out.size(), 0); // this needs to be maxa longer
 	  
 	std::fill(g_out.begin(), g_out.end(), 0); 
 
@@ -87,7 +83,7 @@ int elsym(const int routing, const vec& b, const ivec& a,
 				if(first[i] != item1_first && first[i] != item2_first)
 				{
 					for (int s=Msc; s>=0;s--)
-						for (int j=last[i];j>=first[i];j--)
+						for (int j=last[i];j>=first[i];j--) //needs if(a[j] +s) <= Msc or a longer g vector
 							g[s+a[j]] += g[s]*b[j];
 
 					Msc+=a[last[i]];
@@ -190,7 +186,8 @@ void Expect( const arma::vec& b, const arma::ivec& a,
 			 const arma::ivec& scoretab,  /* out */ arma::vec& E)
 {
 	const int nb = nmod.n_elem;
-	const int len_g = max(bmax) + 1;
+	const int maxA = max(a);
+	const int len_g = max(bmax) + 1 + maxA;
 	
 	// bookkeeping
 	ivec cbmax(nb+1), bnit(nb, fill::zeros), cbnit(nb+1), cbmod(nb+1);
@@ -223,9 +220,9 @@ void Expect( const arma::vec& b, const arma::ivec& a,
 			for(int j=bfirst[i]; j<= blast[i]; j++)
 			{
 			  elsym(brouting[bi], b, a, bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
-           mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
-            mnit.memptr() + cbmod[bi],  gi, bfirst[i], a[j]);
-			  
+					mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
+					mnit.memptr() + cbmod[bi],  gi, bfirst[i], a[j]);
+				  
 				for (int s = a[j]; s <= bmax[bi]; s++) 
 				{
 				  if(g[s]>0)
@@ -239,7 +236,6 @@ void Expect( const arma::vec& b, const arma::ivec& a,
 
 }
 
-
 // [[Rcpp::export]]
 void NR( const arma::vec& b, const arma::ivec& a, 
              arma::ivec& bfirst, arma::ivec& blast, const arma::ivec& bmax,
@@ -248,8 +244,8 @@ void NR( const arma::vec& b, const arma::ivec& a,
 			 const arma::ivec& scoretab,  /* out */ arma::vec& E, arma::mat& H)
 {
 	const int nb = nmod.n_elem;
-	const int len_g = max(bmax) + 1;// if crashes + maxa
-	double tmp;
+	const int maxA = max(a);
+	const int len_g = max(bmax) + 1 + maxA;
 	
 	// bookkeeping
 	ivec cbmax(nb+1), bnit(nb, fill::zeros), cbnit(nb+1), cbmod(nb+1);
@@ -268,80 +264,83 @@ void NR( const arma::vec& b, const arma::ivec& a,
 	
 	E.zeros();
 	H.zeros();
-	std::vector<long double> g(len_g), gi(len_g), gk(len_g), gik(len_g);
-	vec cc(len_g, fill::zeros);
-		
-	for(int bi=0; bi<nb; bi++)
+#pragma omp parallel
 	{
-		elsym(brouting[bi],b, a, bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
-				mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
-				mnit.memptr() + cbmod[bi], g);
-
-		for(int i=cbnit[bi]; i<cbnit[bi+1]; i++)
+		std::vector<long double> g(len_g), gi(len_g), gk(len_g), gik(len_g);
+		vec cc(len_g, fill::zeros);
+#pragma omp for reduction(+: E, H)
+		for(int bi=0; bi<nb; bi++)
 		{
-			for (int j=bfirst[i]; j<=blast[i]; j++)
+			elsym(brouting[bi],b, a, bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
+					mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
+					mnit.memptr() + cbmod[bi], g);
+
+			for(int i=cbnit[bi]; i<cbnit[bi+1]; i++)
 			{
-				elsym(brouting[bi], b, a,  bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
-						mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
-						mnit.memptr() + cbmod[bi], gi, bfirst[i], a[j]);
-				
-				for (int s = a[j]; s <= bmax[bi]; s++)
+				for (int j=bfirst[i]; j<=blast[i]; j++)
 				{
-					if(g[s]>0)
-					{
-						tmp = b[j] * (gi[s-a[j]]/g[s]);
-						cc[s] = scoretab[cbmax[bi]+s] * tmp;
-						E[j] += cc[s]; 
-						H.at(j,j) += cc[s] * (1-tmp);
-					}
-				}
-				
-				// between categories of the same item
-				for (int k = (j+1); k <= blast[i]; k++)
-				{
-					for (int s = a[k]; s <= bmax[bi]; s++)
+					elsym(brouting[bi], b, a,  bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
+							mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
+							mnit.memptr() + cbmod[bi], gi, bfirst[i], a[j]);
+					
+					for (int s = a[j]; s <= bmax[bi]; s++)
 					{
 						if(g[s]>0)
-							H.at(k,j) -= cc[s] * b[k] * (gi[s-a[k]]/g[s]);						
-					}
-					H.at(j,k) = H.at(k,j);
-				}
-				
-				for (int k=i+1; k < cbnit[bi+1]; k++)
-				{
-					for (int l=bfirst[k]; l<=blast[k];l++)
-					{
-						elsym(brouting[bi], b, a,  bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
-								mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
-								mnit.memptr() + cbmod[bi], gk, bfirst[k],a[l]);
-					  
-						int success = elsym(brouting[bi], b, a,  bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
-											mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
-											mnit.memptr() + cbmod[bi], gik, bfirst[i], a[j], bfirst[k], a[l]);
-						if(success>=0)
-						for (int s=0;s<=bmax[bi];s++)
 						{
-							if (g[s]>0)
+							double tmp = b[j] * (gi[s-a[j]]/g[s]);
+							cc[s] = scoretab[cbmax[bi]+s] * tmp;
+							E[j] += cc[s]; 
+							H.at(j,j) += cc[s] * (1-tmp);
+						}
+					}
+					
+					// between categories of the same item
+					for (int k = (j+1); k <= blast[i]; k++)
+					{
+						for (int s = a[k]; s <= bmax[bi]; s++)
+						{
+							if(g[s]>0)
+								H.at(k,j) -= cc[s] * b[k] * (gi[s-a[k]]/g[s]);						
+						}
+						H.at(j,k) = H.at(k,j);
+					}
+					
+					for (int k=i+1; k < cbnit[bi+1]; k++)
+					{
+						for (int l=bfirst[k]; l<=blast[k];l++)
+						{
+							elsym(brouting[bi], b, a,  bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
+									mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
+									mnit.memptr() + cbmod[bi], gk, bfirst[k],a[l]);
+						  
+							int success = elsym(brouting[bi], b, a,  bfirst.memptr() + cbnit[bi], blast.memptr() + cbnit[bi], bnit[bi], 
+												mod_min.memptr() + cbmod[bi], mod_max.memptr() + cbmod[bi], nmod[bi],
+												mnit.memptr() + cbmod[bi], gik, bfirst[i], a[j], bfirst[k], a[l]);
+							if(success>=0)
+							for (int s=0;s<=bmax[bi];s++)
 							{
-								if (s >= a[j]+a[l]) 
+								if (g[s]>0)
 								{
-									H.at(l,j) +=  scoretab[cbmax[bi]+s] * (gik[s-a[j]-a[l]]/g[s]) * b[j]*b[l];
-								}
-								if (s >= a[j] && s >= a[l]) // hier gaat em mis waarschijnlijk 
-								{
-									H.at(l,j) -= cc[s] * b[l] * (gk[s-a[l]] / g[s]);//scoretab[cbmax[bi]+s] * b[j] * b[l] * ((gi[s-a[j]]/g[s])*(gk[s-a[l]]/g[s]));
+									if (s >= a[j]+a[l]) 
+									{
+										H.at(l,j) +=  scoretab[cbmax[bi]+s] * (gik[s-a[j]-a[l]]/g[s]) * b[j]*b[l];
+									}
+									if (s >= a[j] && s >= a[l]) 
+									{
+										H.at(l,j) -= cc[s] * b[l] * (gk[s-a[l]] / g[s]);//scoretab[cbmax[bi]+s] * b[j] * b[l] * ((gi[s-a[j]]/g[s])*(gk[s-a[l]]/g[s]));
+									}
 								}
 							}
+							H.at(j,l) = H.at(l,j);
 						}
-						H.at(j,l) = H.at(l,j);
 					}
-				}
-			}		
+				}		
+			}
 		}
 	}
-	// there seems to be no sum of H with it's transpose like in dexter, is this caused by omission of 0 cat?
 }
 
+
 // [[Rcpp::export]]
 void dirichlet(const arma::vec& alpha, arma::vec& out)
 {
@@ -373,7 +372,8 @@ arma::mat calibrate_Bayes(const arma::ivec& a, const arma::ivec& first, const ar
 	const int nIter = ndraws * step + from;
 
 	const int nb = nmod.n_elem;
-	const int len_g = max(bmax) + 1;
+	const int maxA = max(a);;
+	const int nscores = max(bmax) + 1;
 	const int nit = first.n_elem;
 	const int max_cat = max(last-first)+1;
 	
@@ -401,16 +401,16 @@ arma::mat calibrate_Bayes(const arma::ivec& a, const arma::ivec& first, const ar
 	// working variables
 	vec y(max_cat), z(nb, fill::zeros);
 	vec bklambda(scoretab.n_elem, fill::zeros);	
-	vec pi_k(len_g, fill::zeros);
+	vec pi_k(nscores, fill::zeros);
 	
 	// set lambda 1 for existing scores
 	for(int k=0; k<nb; k++)
 		for(int s=bmin[k]; s<=bmax[k]; s++)
 			bklambda[cbmax[k]+s] = 1;
 	
-	std::vector<long double> g(len_g);
+	std::vector<long double> g(nscores + maxA);
 	
-	vec fpwr(len_g);
+	vec fpwr(nscores);
 	fpwr[0] = 1;
 	
 	//output
@@ -505,66 +505,67 @@ arma::mat calibrate_Bayes(const arma::ivec& a, const arma::ivec& first, const ar
 
 
 // to do: possible scores (in range)
-// to do: void may be slightly faster
+
 //[[Rcpp::export]]
 arma::mat  ittotmat_mst( const arma::vec& b, const arma::ivec& a, const arma::vec& c, 
              arma::ivec& first, arma::ivec& last, 
 			 const int bmin, const int bmax, const int nmod, const int brouting,
 			 arma::ivec& mnit, const arma::ivec& mod_min, const arma::ivec& mod_max)
 {
-	
+	// zou last.n_elem verkeerd kunnen zijn in aanroep?
 	const int nI = last.n_elem;
 	const int npar = accu(last-first) + nI;
-	const int nscores = bmax+1;
+	const int nscores = bmax+1, maxA=max(a);
 	const vec logb = log(b);
 	const vec alogc = a % log(c);
-	int indx;
 	  
-	mat pi(npar, nscores, fill::zeros);	  
- 
-		std::vector<long double> g(nscores), gi(nscores);
+	mat pi(npar, nscores, fill::zeros);	  	
+	
+#pragma omp parallel
+	{
+		std::vector<long double> g(nscores + maxA), gi(nscores + maxA);
 		vec eta(npar);
-
+#pragma omp for
 		for (int s = bmin; s <= bmax; s++)
 		{
 			//if(ps[s] == 1)
 			//{
 				int k = 0; 
 				eta = exp(logb + s * alogc);
-
+				
 				elsym(brouting, eta, a,  first.memptr(), last.memptr(), nI, 
 						mod_min.memptr(), mod_max.memptr(), nmod,
 						mnit.memptr(), g);
-
+				
 				for (int it = 0; it < nI; it++)
 				{
 					for (int j = first[it]; j <= last[it]; j++) 
-					{
-						elsym(brouting, eta, a,  first.memptr(), last.memptr(), nI, 
-								mod_min.memptr(), mod_max.memptr(), nmod,mnit.memptr(), gi, first[it],a[j]);
-						indx = s-a[j];
+					{					
+						int indx = s-a[j];
 						if ( indx >= 0 && indx < (nscores - a[last[it]])) 
-							//pi.at(k,s) = exp(log(eta[j]) + log(gi[indx]) - log(g[s]));
+						{
+							elsym(brouting, eta, a,  first.memptr(), last.memptr(), nI, 
+								mod_min.memptr(), mod_max.memptr(), nmod,mnit.memptr(), gi, first[it],a[j]);
 							pi.at(k,s) = eta[j] * gi[indx]/g[s];
+						}
 						k++;
 					}
 				}
 			//}
 		}
+	}
 	return pi;
 }
 
 
 
-// might change to log scale
-
 // for one booklet
 //[[Rcpp::export]]
 arma::vec elsym_C(const int routing, const arma::vec& b, const arma::ivec& a, arma::ivec& first, arma::ivec& last, 
 					arma::ivec& mod_min, arma::ivec& mod_max, arma::ivec& mnit, const int max_score,
 					const int item1_first=-1, const int aij=1, const int item2_first=-1, const int akl=1)
 {
-	std::vector<long double> g(max_score+1);
+	std::vector<long double> g(max_score+1+max(a));
 	vec out(max_score+1);
 	elsym(routing, b, a, first.memptr(), last.memptr(), first.n_elem, mod_min.memptr(), mod_max.memptr(), 
        mnit.n_elem, mnit.memptr(), g, item1_first, aij, item2_first, akl);
@@ -584,7 +585,9 @@ arma::vec prof_enorm(const arma::vec& b, const arma::ivec& a, arma::ivec& first,
 	const int nmod = mnit.n_elem;
 	int idx=0, mA=0, mB=0;
 	
-	std::vector<long double> gA(max_score+1), gB(max_score+1);
+	//working memory size, take early finish booklets into account
+	const int sz = 1 + accu(a.elem(conv_to<uvec>::from(last)));
+	std::vector<long double> gA(sz), gB(sz);
 	
 	ivec cnit(nmod+1);
 	cnit[0] = 0;
